@@ -170,6 +170,7 @@ pub fn functions() -> HashSet<&'static str>
         "normD",
         "normP",
         "piecewise",
+        "pw",
         "is_prime",
         "isprime",
         "dice",
@@ -195,8 +196,9 @@ pub fn functions() -> HashSet<&'static str>
         "length",
         "arclength",
         "roll",
+        "erfi",
     ]
     .iter()
     .cloned()
     .collect::<HashSet<&str>>()
-}
\ No newline at end of file
+}
