@@ -4,7 +4,10 @@ use crate::{
         NumStr::{Matrix, Num, Str, Vector},
     },
     math::do_math,
-    misc::prompt,
+    misc::{
+        get_terminal_dimensions_pixel, place_funcvar, place_funcvarxy, place_var, place_varxy,
+        prompt,
+    },
     Colors, Options,
 };
 use gnuplot::{AxesCommon, Caption, Color, Figure, Fix, PointSymbol};
@@ -38,9 +41,9 @@ pub fn graph(
         let mut d2_or_d3 = (false, false);
         let mut lines = false;
         let mut handles = Vec::new();
-        for func in func
+        for (func, input) in func.iter().zip(input.iter())
         {
-            handles.push(get_data(colors.clone(), func));
+            handles.push(get_data(colors.clone(), func.clone(), input.clone()));
         }
         let mut i = 0;
         #[allow(clippy::explicit_counter_loop)]
@@ -80,7 +83,7 @@ pub fn graph(
         if d2_or_d3.0 == d2_or_d3.1
         {
             print!(
-                "\x1b[G\x1b[Kcant graph 2d and 3d\n\x1b[G{}",
+                "\x1b[G\x1b[Kcant graph 2d and 3d\x1b[G\n{}",
                 prompt(options, &colors)
             );
             stdout().flush().unwrap();
@@ -90,7 +93,7 @@ pub fn graph(
         {
             if lines
             {
-                if Options::default().yr == options.yr
+                if Options::default().xr == options.xr
                 {
                     options.xr = (
                         points2d.iter().fold(f64::MAX, |min, x| {
@@ -156,6 +159,13 @@ pub fn graph(
                     Some((Fix((options.yr.1 - options.yr.0) / options.ticks), 1)),
                 )
             };
+            if options.scale_graph
+            {
+                let (x, y) = get_terminal_dimensions_pixel();
+                let w = y as f64 / x as f64 * (options.xr.1 - options.xr.0)
+                    / (options.yr.1 - options.yr.0);
+                options.yr = (w * options.yr.0, w * options.yr.1);
+            }
             if options.lines || lines
             {
                 fg.axes2d()
@@ -409,7 +419,7 @@ pub fn graph(
         {
             if lines
             {
-                if Options::default().yr == options.yr
+                if Options::default().xr == options.xr
                 {
                     options.xr = (
                         points3d.iter().fold(f64::MAX, |min, x| {
@@ -903,21 +913,30 @@ pub fn graph(
         }
         if let Some(time) = watch
         {
-            print!("\x1b[G\x1b[K{}ms\n\x1b[G", time.elapsed().as_millis(),);
-            stdout().flush().unwrap();
+            println!("\x1b[G\x1b[K{}ms\x1b[G\n", time.elapsed().as_millis());
         }
         if fg.show().is_err()
         {
-            print!("\x1b[G\x1b[Kno gnuplot\n\x1b[G{}", prompt(options, &colors));
+            print!("\x1b[G\x1b[Kno gnuplot\x1b[G\n{}", prompt(options, &colors));
             stdout().flush().unwrap();
         }
     })
 }
 #[allow(clippy::type_complexity)]
 pub fn get_list_2d(
-    func: (Vec<NumStr>, Vec<(String, Vec<NumStr>)>, Options),
+    mut func: (Vec<NumStr>, Vec<(String, Vec<NumStr>)>, Options),
 ) -> ([[Vec<f64>; 2]; 2], [Vec<f64>; 2], (bool, bool))
 {
+    {
+        let temp = func.1.clone();
+        for i in func.1.iter_mut()
+        {
+            if let Ok(n) = do_math(i.1.clone(), func.2, temp.clone())
+            {
+                i.1 = vec![n];
+            }
+        }
+    }
     if let Num(n) = &func.0[0]
     {
         if func.0.len() == 1 && n.is_zero()
@@ -1132,12 +1151,17 @@ pub fn get_list_2d(
             {}
         }
     }
-    if !zero.0
+    if !zero.0 && zero.1
     {
         data[0][1] = Vec::new();
         data3d[0] = Vec::new();
     }
-    if !zero.1
+    if !zero.1 && zero.0
+    {
+        data[1][1] = Vec::new();
+        data3d[1] = Vec::new();
+    }
+    if !zero.1 && !zero.0
     {
         data[1][1] = Vec::new();
         data3d[1] = Vec::new();
@@ -1146,9 +1170,19 @@ pub fn get_list_2d(
 }
 #[allow(clippy::type_complexity)]
 pub fn get_list_3d(
-    func: (Vec<NumStr>, Vec<(String, Vec<NumStr>)>, Options),
+    mut func: (Vec<NumStr>, Vec<(String, Vec<NumStr>)>, Options),
 ) -> ([[Vec<f64>; 3]; 2], (bool, bool), bool)
 {
+    {
+        let temp = func.1.clone();
+        for i in func.1.iter_mut()
+        {
+            if let Ok(n) = do_math(i.1.clone(), func.2, temp.clone())
+            {
+                i.1 = vec![n];
+            }
+        }
+    }
     if let Num(n) = &func.0[0]
     {
         if func.0.len() == 1 && n.is_zero()
@@ -1376,89 +1410,25 @@ pub fn get_list_3d(
             }
         }
     }
-    if !zero.0
+    if !zero.0 && zero.1
     {
         data[0][2] = Vec::new();
     }
-    if !zero.1
+    if !zero.1 && zero.0
+    {
+        data[1][2] = Vec::new();
+    }
+    if !zero.1 && !zero.0
     {
         data[1][2] = Vec::new();
     }
     (data, zero, d2)
 }
-fn place_funcvarxy(
-    mut funcvar: Vec<(String, Vec<NumStr>)>,
-    num: NumStr,
-) -> Vec<(String, Vec<NumStr>)>
-{
-    for i in funcvar.iter_mut()
-    {
-        for j in i.1.iter_mut()
-        {
-            if let Str(s) = j
-            {
-                if matches!(s.as_str(), "x" | "y")
-                {
-                    *j = num.clone()
-                }
-            }
-        }
-    }
-    funcvar
-}
-fn place_funcvar(
-    mut funcvar: Vec<(String, Vec<NumStr>)>,
-    var: &str,
-    num: NumStr,
-) -> Vec<(String, Vec<NumStr>)>
-{
-    for i in funcvar.iter_mut()
-    {
-        for j in i.1.iter_mut()
-        {
-            if let Str(s) = j
-            {
-                if s == var
-                {
-                    *j = num.clone()
-                }
-            }
-        }
-    }
-    funcvar
-}
-fn place_varxy(mut func: Vec<NumStr>, num: NumStr) -> Vec<NumStr>
-{
-    for i in func.iter_mut()
-    {
-        if let Str(s) = i
-        {
-            if matches!(s.as_str(), "x" | "y")
-            {
-                *i = num.clone()
-            }
-        }
-    }
-    func
-}
-pub fn place_var(mut func: Vec<NumStr>, var: &str, num: NumStr) -> Vec<NumStr>
-{
-    for i in func.iter_mut()
-    {
-        if let Str(s) = i
-        {
-            if s == var
-            {
-                *i = num.clone()
-            }
-        }
-    }
-    func
-}
-fn fail(options: Options, colors: &Colors)
+fn fail(options: Options, colors: &Colors, input: String)
 {
     print!(
-        "\x1b[G\x1b[KNo data to plot\n\x1b[G{}",
+        "\x1b[G\x1b[KNo data to plot for {}\x1b[G\n{}",
+        input,
         prompt(options, colors)
     );
     stdout().flush().unwrap();
@@ -1467,6 +1437,7 @@ fn fail(options: Options, colors: &Colors)
 fn get_data(
     colors: Colors,
     func: (Vec<NumStr>, Vec<(String, Vec<NumStr>)>, Options),
+    input: String,
 ) -> JoinHandle<(
     (bool, bool),
     (bool, bool),
@@ -1495,7 +1466,7 @@ fn get_data(
                 Ok(n) => n,
                 _ =>
                 {
-                    fail(func.2, &colors);
+                    fail(func.2, &colors, input);
                     return (
                         (false, false),
                         (false, false),
@@ -1510,10 +1481,29 @@ fn get_data(
                 Num(n) =>
                 {
                     d2_or_d3.0 = true;
-                    (points2d, _, re_or_im) = get_list_2d((vec![Num(n)], Vec::new(), func.2));
+                    let change = (func.2.xr.1 - func.2.xr.0) / func.2.samples_2d as f64;
+                    let im = n.imag().to_f64();
+                    let re = n.real().to_f64();
+                    for i in 0..func.2.samples_2d
+                    {
+                        if re != 0.0
+                        {
+                            points2d[0][0].push(func.2.xr.0 + change * i as f64);
+                            points2d[0][1].push(re);
+                        }
+                        if im != 0.0
+                        {
+                            if re == 0.0
+                            {
+                                points2d[1][0].push(func.2.xr.0 + change * i as f64);
+                            }
+                            points2d[1][1].push(im);
+                        }
+                    }
+                    re_or_im = (re != 0.0, im != 0.0);
                     if points2d[0][1].is_empty() && points2d[1][1].is_empty()
                     {
-                        fail(func.2, &colors);
+                        fail(func.2, &colors, input);
                         return (
                             (false, false),
                             (false, false),
@@ -1564,7 +1554,7 @@ fn get_data(
                             points2d[1][1].extend(points2dtemp[1][1].clone());
                             if points2d[0][1].is_empty() && points2d[1][1].is_empty()
                             {
-                                fail(func.2, &colors);
+                                fail(func.2, &colors, input);
                                 return (
                                     (false, false),
                                     (false, false),
@@ -1729,7 +1719,7 @@ fn get_data(
             }
             else if points2d[0][1].is_empty() && points2d[1][1].is_empty()
             {
-                fail(func.2, &colors);
+                fail(func.2, &colors, input);
                 return (
                     (false, false),
                     (false, false),
@@ -1781,7 +1771,7 @@ fn get_data(
             }
             else if points3d[0][2].is_empty() && points3d[1][2].is_empty()
             {
-                fail(func.2, &colors);
+                fail(func.2, &colors, input);
                 return (
                     (false, false),
                     (false, false),
@@ -1809,4 +1799,4 @@ fn get_data(
         }
         (d2_or_d3, re_or_im, lines, false, points2d, points3d)
     })
-}
\ No newline at end of file
+}
