@@ -16,7 +16,7 @@ use crate::{
     graph::graph,
     load_vars::{add_var, get_cli_vars, get_file_vars, get_vars, set_commands_or_vars},
     math::do_math,
-    misc::{clearln, convert, get_terminal_width, prompt, read_single_char, write},
+    misc::{clearln, convert, get_terminal_dimensions, prompt, read_single_char, write},
     options::{arg_opts, commands, file_opts, silent_commands},
     parse::input_var,
     print::{print_answer, print_concurrent},
@@ -35,6 +35,11 @@ use std::{
 //rpn
 //matrix exponentiation
 //support units properly, add a part to the Num struct where it just stores the unit which then can be dealt with in complex or smth
+//maybe assume brackets for say log(10x^(2,4) to log(10x^(2),4) if possible
+//dont use strings in math.rs
+//dont clear terminal first, write the line, \x1b[K, then on the last line \x1[b[J
+//remove \x1b[A repeats and \x08 and said repeats \x1b[D
+//do as much as you can before graphing
 #[derive(Clone)]
 pub struct Variable
 {
@@ -143,6 +148,7 @@ pub struct Options
     var_multiply: bool,
     interactive: bool,
     surface: bool,
+    scale_graph: bool,
 }
 impl Default for Options
 {
@@ -182,10 +188,11 @@ impl Default for Options
             depth: false,
             flat: false,
             graph: true,
-            slowcheck: 250,
+            slowcheck: 300,
             var_multiply: false,
             interactive: true,
             surface: false,
+            scale_graph: true,
         }
     }
 }
@@ -520,7 +527,8 @@ fn main()
                 {
                     '\n' | '\x14' | '\x09' | '\x06' =>
                     {
-                        end = start + get_terminal_width() - if options.prompt { 3 } else { 1 };
+                        end = start + get_terminal_dimensions().0
+                            - if options.prompt { 3 } else { 1 };
                         if end > input.len()
                         {
                             end = input.len()
@@ -538,7 +546,7 @@ fn main()
                                 colors.clone(),
                                 start,
                                 end,
-                                (long || !options.real_time_output) && c == '\n',
+                                c == '\n',
                             );
                         }
                         if !input.is_empty()
@@ -557,12 +565,12 @@ fn main()
                             }
                             else
                             {
-                                print!("\n\x1b[G\x1b[J");
+                                print!("\x1b[G\n\x1b[J");
                             }
                             terminal::disable_raw_mode().unwrap();
                             std::process::exit(0);
                         }
-                        print!("\n\x1b[G\x1b[K");
+                        print!("\x1b[G\n\x1b[K");
                         if c == '\x09'
                         {
                             graphable = false;
@@ -581,7 +589,8 @@ fn main()
                             placement -= 1;
                             input.remove(placement);
                         }
-                        end = start + get_terminal_width() - if options.prompt { 3 } else { 1 };
+                        end = start + get_terminal_dimensions().0
+                            - if options.prompt { 3 } else { 1 };
                         if end > input.len()
                         {
                             end = input.len()
@@ -610,7 +619,7 @@ fn main()
                             {
                                 slow = true;
                                 print!(
-                                        "\n\x1b[G\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
+                                        "\x1b[G\n\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
                                         input.len() + if options.prompt { 2 } else { 0 }
                                     );
                             }
@@ -644,7 +653,8 @@ fn main()
                         {
                             input.remove(placement);
                         }
-                        end = start + get_terminal_width() - if options.prompt { 3 } else { 1 };
+                        end = start + get_terminal_dimensions().0
+                            - if options.prompt { 3 } else { 1 };
                         if end > input.len()
                         {
                             end = input.len()
@@ -673,7 +683,7 @@ fn main()
                             {
                                 slow = true;
                                 print!(
-                                        "\n\x1b[G\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
+                                        "\x1b[G\n\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
                                         input.len() + if options.prompt { 2 } else { 0 }
                                     );
                             }
@@ -701,7 +711,7 @@ fn main()
                         //end
                         placement = input.len();
                         end = input.len();
-                        start = if get_terminal_width() - if options.prompt { 3 } else { 1 }
+                        start = if get_terminal_dimensions().0 - if options.prompt { 3 } else { 1 }
                             > input.len()
                         {
                             0
@@ -709,7 +719,7 @@ fn main()
                         else
                         {
                             input.len()
-                                - (get_terminal_width() - if options.prompt { 3 } else { 1 })
+                                - (get_terminal_dimensions().0 - if options.prompt { 3 } else { 1 })
                         };
                         if options.real_time_output && !slow
                         {
@@ -727,7 +737,7 @@ fn main()
                             {
                                 slow = true;
                                 print!(
-                                        "\n\x1b[G\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
+                                        "\x1b[G\n\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
                                         input.len() + if options.prompt { 2 } else { 0 }
                                     );
                             }
@@ -759,7 +769,7 @@ fn main()
                             {
                                 slow = true;
                                 print!(
-                                        "\n\x1b[G\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
+                                        "\x1b[G\n\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
                                         input.len() + if options.prompt { 2 } else { 0 }
                                     );
                             }
@@ -791,7 +801,7 @@ fn main()
                             {
                                 slow = true;
                                 print!(
-                                        "\n\x1b[G\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
+                                        "\x1b[G\n\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
                                         input.len() + if options.prompt { 2 } else { 0 }
                                     );
                             }
@@ -825,7 +835,7 @@ fn main()
                             {
                                 slow = true;
                                 print!(
-                                        "\n\x1b[G\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
+                                        "\x1b[G\n\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
                                         input.len() + if options.prompt { 2 } else { 0 }
                                     );
                             }
@@ -859,7 +869,7 @@ fn main()
                                 {
                                     slow = true;
                                     print!(
-                                            "\n\x1b[G\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
+                                            "\x1b[G\n\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
                                             input.len() + if options.prompt { 2 } else { 0 }
                                         );
                                 }
@@ -890,7 +900,7 @@ fn main()
                             {
                                 slow = true;
                                 print!(
-                                        "\n\x1b[G\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
+                                        "\x1b[G\n\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
                                         input.len() + if options.prompt { 2 } else { 0 }
                                     );
                             }
@@ -906,14 +916,14 @@ fn main()
                         //home
                         placement = 0;
                         start = 0;
-                        end = if get_terminal_width() - if options.prompt { 3 } else { 1 }
+                        end = if get_terminal_dimensions().0 - if options.prompt { 3 } else { 1 }
                             > input.len()
                         {
                             input.len()
                         }
                         else
                         {
-                            get_terminal_width() - if options.prompt { 3 } else { 1 }
+                            get_terminal_dimensions().0 - if options.prompt { 3 } else { 1 }
                         };
                         if options.real_time_output && !slow
                         {
@@ -931,7 +941,7 @@ fn main()
                             {
                                 slow = true;
                                 print!(
-                                        "\n\x1b[G\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
+                                        "\x1b[G\n\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
                                         input.len() + if options.prompt { 2 } else { 0 }
                                     );
                             }
@@ -953,7 +963,7 @@ fn main()
                         input = lines[i].clone().chars().collect::<Vec<char>>();
                         placement = input.len();
                         end = input.len();
-                        start = if get_terminal_width() - if options.prompt { 3 } else { 1 }
+                        start = if get_terminal_dimensions().0 - if options.prompt { 3 } else { 1 }
                             > input.len()
                         {
                             0
@@ -961,7 +971,7 @@ fn main()
                         else
                         {
                             input.len()
-                                - (get_terminal_width() - if options.prompt { 3 } else { 1 })
+                                - (get_terminal_dimensions().0 - if options.prompt { 3 } else { 1 })
                         };
                         if options.real_time_output
                         {
@@ -979,7 +989,7 @@ fn main()
                             if slow
                             {
                                 print!(
-                                        "\n\x1b[G\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
+                                        "\x1b[G\n\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
                                         input.len() + if options.prompt { 2 } else { 0 }
                                     );
                             }
@@ -1004,7 +1014,7 @@ fn main()
                         }
                         placement = input.len();
                         end = input.len();
-                        start = if get_terminal_width() - if options.prompt { 3 } else { 1 }
+                        start = if get_terminal_dimensions().0 - if options.prompt { 3 } else { 1 }
                             > input.len()
                         {
                             0
@@ -1012,7 +1022,7 @@ fn main()
                         else
                         {
                             input.len()
-                                - (get_terminal_width() - if options.prompt { 3 } else { 1 })
+                                - (get_terminal_dimensions().0 - if options.prompt { 3 } else { 1 })
                         };
                         if options.real_time_output
                         {
@@ -1030,7 +1040,7 @@ fn main()
                             if slow
                             {
                                 print!(
-                                        "\n\x1b[G\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
+                                        "\x1b[G\n\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
                                         input.len() + if options.prompt { 2 } else { 0 }
                                     );
                             }
@@ -1047,7 +1057,8 @@ fn main()
                         {
                             start -= 1;
                             placement -= 1;
-                            end = start + get_terminal_width() - if options.prompt { 3 } else { 1 };
+                            end = start + get_terminal_dimensions().0
+                                - if options.prompt { 3 } else { 1 };
                             if end > input.len()
                             {
                                 end = input.len()
@@ -1064,7 +1075,8 @@ fn main()
                     '\x1C' =>
                     {
                         //right
-                        end = start + get_terminal_width() - if options.prompt { 3 } else { 1 };
+                        end = start + get_terminal_dimensions().0
+                            - if options.prompt { 3 } else { 1 };
                         if end > input.len()
                         {
                             end = input.len()
@@ -1167,7 +1179,9 @@ fn main()
                     {
                         input.insert(placement, c);
                         placement += 1;
-                        end = start + get_terminal_width() - if options.prompt { 3 } else { 1 } + 1;
+                        end = start + get_terminal_dimensions().0
+                            - if options.prompt { 3 } else { 1 }
+                            + 1;
                         if end > input.len()
                         {
                             end = input.len()
@@ -1204,7 +1218,7 @@ fn main()
                             {
                                 slow = true;
                                 print!(
-                                        "\n\x1b[G\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
+                                        "\x1b[G\n\x1b[Jtoo slow, will print on enter\x1b[A\x1b[G\x1b[{}C",
                                         input.len() + if options.prompt { 2 } else { 0 }
                                     );
                             }
@@ -1274,7 +1288,7 @@ fn main()
             {
                 if !s.is_empty()
                 {
-                    print!("\x1b[G\x1b[K{}\x1b[B\x1b[G{}", s, prompt(options, &colors));
+                    print!("\x1b[G\x1b[K{}\n\x1b[G{}", s, prompt(options, &colors));
                 }
                 else
                 {
@@ -1343,7 +1357,7 @@ fn main()
                                     )
                                     {
                                         print!(
-                                            "\x1b[A\x1b[G\x1b[K{}\x1b[B\x1b[G{}",
+                                            "\x1b[A\x1b[G\x1b[K{}\n\x1b[G{}",
                                             s,
                                             prompt(options, &colors)
                                         );
@@ -1383,4 +1397,4 @@ fn main()
             }
         }
     }
-}
\ No newline at end of file
+}
