@@ -1,4 +1,9 @@
-use crate::{functions::functions, Colors, Options};
+use crate::{
+    complex::{NumStr, NumStr::Str},
+    functions::functions,
+    math::do_math,
+    Colors, Options,
+};
 use crossterm::event::{read, Event, KeyCode, KeyEvent, KeyModifiers};
 #[cfg(unix)]
 use libc::{ioctl, winsize, STDOUT_FILENO, TIOCGWINSZ};
@@ -6,32 +11,32 @@ use std::{fs::File, io::Write};
 #[cfg(not(unix))]
 use term_size::dimensions;
 #[cfg(unix)]
-pub fn get_terminal_width() -> usize
+pub fn get_terminal_dimensions() -> (usize, usize)
 {
     unsafe {
         let mut size: winsize = std::mem::zeroed();
         if ioctl(STDOUT_FILENO, TIOCGWINSZ, &mut size) == 0 && size.ws_col != 0
         {
-            size.ws_col as usize
+            (size.ws_col as usize, size.ws_row as usize)
         }
         else
         {
-            80
+            (80, 80)
         }
     }
 }
 #[cfg(unix)]
-pub fn get_terminal_height() -> usize
+pub fn get_terminal_dimensions_pixel() -> (usize, usize)
 {
     unsafe {
         let mut size: winsize = std::mem::zeroed();
         if ioctl(STDOUT_FILENO, TIOCGWINSZ, &mut size) == 0 && size.ws_col != 0
         {
-            size.ws_row as usize
+            (size.ws_xpixel as usize, size.ws_ypixel as usize)
         }
         else
         {
-            80
+            (80, 80)
         }
     }
 }
@@ -144,7 +149,7 @@ pub fn convert(c: &char) -> char
 {
     let valid_chars = [
         '+', '^', '(', ')', '.', '=', ',', '#', '|', '&', '!', '%', '_', '<', '>', ' ', '[', ']',
-        '{', '}', '√', '∛', '⁻', 'ⁱ', '`', '±', '∞', ';', '\'',
+        '{', '}', '√', '∛', '⁻', 'ⁱ', '`', '±', '∞', ';', ':', '\'',
     ];
     match c
     {
@@ -250,6 +255,15 @@ pub fn clearln(input: &[char], start: usize, end: usize, options: Options, color
         if options.color { "\x1b[0m" } else { "" }
     );
 }
+pub fn clear(input: &[char], start: usize, end: usize, options: Options, colors: &Colors)
+{
+    print!(
+        "\x1b[G\x1b[J{}{}{}",
+        prompt(options, colors),
+        to_output(&input[start..end], options.color, colors),
+        if options.color { "\x1b[0m" } else { "" }
+    );
+}
 pub fn to_output(input: &[char], color: bool, colors: &Colors) -> String
 {
     if color
@@ -264,6 +278,11 @@ pub fn to_output(input: &[char], color: bool, colors: &Colors) -> String
             let c = input[i];
             match c
             {
+                '#' =>
+                {
+                    count = 0;
+                    output.push(c)
+                }
                 '\x1b' =>
                 {
                     ignore = true;
@@ -346,9 +365,9 @@ pub fn handle_err(
 )
 {
     print!(
-        "\x1b[J\n\x1b[G{}{}\x1b[A\x1b[G\x1b[K{}{}{}",
+        "\x1b[J\x1b[G\n{}{}\x1b[A\x1b[G\x1b[K{}{}{}",
         err,
-        "\x1b[A".repeat(err.len().div_ceil(get_terminal_width()) - 1),
+        "\x1b[A".repeat(err.len().div_ceil(get_terminal_dimensions().0) - 1),
         prompt(options, colors),
         to_output(&input[start..end], options.color, colors),
         if options.color { "\x1b[0m" } else { "" },
@@ -380,3 +399,150 @@ pub fn prompt(options: Options, colors: &Colors) -> String
         String::new()
     }
 }
+pub fn place_funcvarxy(
+    mut funcvar: Vec<(String, Vec<NumStr>)>,
+    num: NumStr,
+) -> Vec<(String, Vec<NumStr>)>
+{
+    for i in funcvar.iter_mut()
+    {
+        if !i.0.contains('(')
+        {
+            for j in i.1.iter_mut()
+            {
+                if let Str(s) = j
+                {
+                    if matches!(s.as_str(), "x" | "y")
+                    {
+                        *j = num.clone()
+                    }
+                }
+            }
+        }
+    }
+    funcvar
+}
+pub fn place_funcvar(
+    mut funcvar: Vec<(String, Vec<NumStr>)>,
+    var: &str,
+    num: NumStr,
+) -> Vec<(String, Vec<NumStr>)>
+{
+    for i in funcvar.iter_mut()
+    {
+        if !i.0.contains('(')
+        {
+            for j in i.1.iter_mut()
+            {
+                if let Str(s) = j
+                {
+                    if s == var
+                    {
+                        *j = num.clone()
+                    }
+                }
+            }
+        }
+    }
+    funcvar
+}
+pub fn place_varxy(mut func: Vec<NumStr>, num: NumStr) -> Vec<NumStr>
+{
+    let mut sum = Vec::new();
+    let mut bracket = 0;
+    let mut i = 0;
+    while func.len() > i
+    {
+        if let Str(s) = &func[i]
+        {
+            if matches!(s.as_str(), "x" | "y") && sum.is_empty()
+            {
+                func[i] = num.clone();
+            }
+            else
+            {
+                match s.as_str()
+                {
+                    "(" => bracket += 1,
+                    ")" => bracket -= 1,
+                    "," if sum.contains(&bracket) =>
+                    {
+                        sum.remove(0);
+                    }
+                    "sum" | "summation" | "prod" | "product" | "Σ" | "Π" | "vec" | "mat" | "D"
+                    | "integrate" | "arclength" | "area" | "length" | "slope" | "lim" | "limit"
+                        if i + 2 < func.len()
+                            && if let Str(s) = &func[i + 2]
+                            {
+                                matches!(s.as_str(), "x" | "y")
+                            }
+                            else
+                            {
+                                false
+                            } =>
+                    {
+                        i += 3;
+                        sum.push(bracket)
+                    }
+                    _ =>
+                    {}
+                }
+            }
+        }
+        i += 1;
+    }
+    func
+}
+pub fn place_var(mut func: Vec<NumStr>, var: &str, num: NumStr) -> Vec<NumStr>
+{
+    let mut sum = Vec::new();
+    let mut bracket = 0;
+    let mut i = 0;
+    while func.len() > i
+    {
+        if let Str(s) = &func[i]
+        {
+            if s == var && sum.is_empty()
+            {
+                func[i] = num.clone();
+            }
+            else
+            {
+                match s.as_str()
+                {
+                    "(" => bracket += 1,
+                    ")" => bracket -= 1,
+                    "," if sum.contains(&bracket) =>
+                    {
+                        sum.remove(0);
+                    }
+                    "sum" | "summation" | "prod" | "product" | "Σ" | "Π" | "vec" | "mat" | "D"
+                    | "integrate" | "arclength" | "area" | "length" | "slope" | "lim" | "limit"
+                        if i + 2 < func.len() && func[i + 2] == Str(var.to_string()) =>
+                    {
+                        i += 3;
+                        sum.push(bracket)
+                    }
+                    _ =>
+                    {}
+                }
+            }
+        }
+        i += 1;
+    }
+    func
+}
+pub fn do_math_with_var(
+    function: Vec<NumStr>,
+    options: Options,
+    func_vars: Vec<(String, Vec<NumStr>)>,
+    var: &str,
+    num: NumStr,
+) -> Result<NumStr, &'static str>
+{
+    do_math(
+        place_var(function, var, num.clone()),
+        options,
+        place_funcvar(func_vars, var, num),
+    )
+}
