@@ -159,14 +159,11 @@ pub fn input_var(
             {
                 output.push(Str('^'.to_string()));
             }
-            output.push(Num(Complex::with_val(
-                options.prec,
-                match Complex::parse(pow.as_bytes())
-                {
-                    Ok(n) => n,
-                    _ => return Err("exponent error"),
-                },
-            ) * Complex::with_val(options.prec, (0, 1))
+            output.push(Num(match Complex::parse(pow.as_bytes())
+            {
+                Ok(n) => n.complete(options.prec),
+                _ => return Err("exponent error"),
+            } * Complex::with_val(options.prec, (0, 1))
                 .pow(Complex::with_val(options.prec, i))));
             pow = String::new();
         }
@@ -576,14 +573,11 @@ pub fn input_var(
                     if !exp.0.is_empty() && exp.1 == *bracket
                     {
                         output.push(Str("^".to_string()));
-                        output.push(Num(Complex::with_val(
-                            options.prec,
-                            match Complex::parse(exp.0.as_bytes())
-                            {
-                                Ok(n) => n,
-                                _ => return Err("exponent error"),
-                            },
-                        )));
+                        output.push(Num(match Complex::parse(exp.0.as_bytes())
+                        {
+                            Ok(n) => n.complete(options.prec),
+                            _ => return Err("exponent error"),
+                        }));
                         exp = (String::new(), 0);
                     }
                 }
@@ -823,7 +817,7 @@ pub fn input_var(
             countv -= 1;
             word.pop();
         }
-        if word == "piecewise" && piecewise == 0
+        if (word == "piecewise" || word == "pw") && piecewise == 0
         {
             piecewise = *bracket as usize + 1;
         }
@@ -842,7 +836,9 @@ pub fn input_var(
                 | "Π"
                 | "D"
                 | "integrate"
-                | "arclength"|"lim"|"limit"
+                | "arclength"
+                | "lim"
+                | "limit"
         ) && chars.len() > i + countv + 1
         {
             let mut place = 0;
@@ -1132,7 +1128,7 @@ pub fn input_var(
                             i = j;
                             continue;
                         }
-                        if piecewise != 0
+                        if blacklist == var.name && piecewise != 0
                         {
                             output.push(Str("@".to_owned() + &var.name.iter().collect::<String>()));
                             i = j + var.name.split(|c| c == &'(').next().unwrap().len();
@@ -1259,6 +1255,16 @@ pub fn input_var(
                                     if (tempgraph && parsed.len() > 1)
                                         || print
                                         || sumrec.iter().any(|c| c.0 == -1)
+                                        || parsed.iter().any(|c| {
+                                            if let Str(s) = c
+                                            {
+                                                sumrec.iter().any(|r| &r.1 == s)
+                                            }
+                                            else
+                                            {
+                                                false
+                                            }
+                                        })
                                     {
                                         let iden =
                                             format!("@{}{}{}{}@", i, func_var, depth, vars.len());
@@ -1402,14 +1408,11 @@ pub fn input_var(
                             if !exp.0.is_empty() && exp.1 == *bracket
                             {
                                 output.push(Str("^".to_string()));
-                                output.push(Num(Complex::with_val(
-                                    options.prec,
-                                    match Complex::parse(exp.0.as_bytes())
-                                    {
-                                        Ok(n) => n,
-                                        _ => return Err("exponent error"),
-                                    },
-                                )));
+                                output.push(Num(match Complex::parse(exp.0.as_bytes())
+                                {
+                                    Ok(n) => n.complete(options.prec),
+                                    _ => return Err("exponent error"),
+                                }));
                                 exp = (String::new(), 0);
                             }
                             i += 1;
@@ -1487,6 +1490,16 @@ pub fn input_var(
                                 if (tempgraph && parsed.len() > 1)
                                     || print
                                     || sumrec.iter().any(|c| c.0 == -1)
+                                    || parsed.iter().any(|c| {
+                                        if let Str(s) = c
+                                        {
+                                            sumrec.iter().any(|r| &r.1 == s)
+                                        }
+                                        else
+                                        {
+                                            false
+                                        }
+                                    })
                                 {
                                     let iden = format!("@{}{}{}{}@", i, l, depth, vars.len());
                                     funcvars.extend(func);
@@ -1732,7 +1745,7 @@ pub fn input_var(
                         }
                         else
                         {
-                            return Err("bad input");
+                            return Err("bad input2");
                         }
                         if scientific
                         {
