@@ -4,10 +4,9 @@ use crate::{
         NumStr::{Matrix, Num, Vector},
     },
     fraction::fraction,
-    get_terminal_width,
     load_vars::set_commands_or_vars,
     math::do_math,
-    misc::{clearln, get_terminal_height, handle_err, no_col, prompt, to_output},
+    misc::{clear, clearln, get_terminal_dimensions, handle_err, no_col, prompt, to_output},
     options::{equal_to, parsed_to_string, silent_commands},
     parse::input_var,
     AngleType::{Degrees, Gradians, Radians},
@@ -27,10 +26,9 @@ pub fn print_concurrent(
     long_output: bool,
 ) -> (usize, bool, bool, bool)
 {
-    if unmodified_input.starts_with(&['#'])
+    if unmodified_input.starts_with(&['#']) || unmodified_input.is_empty()
     {
-        print!("\x1b[J");
-        clearln(unmodified_input, start, end, options, &colors);
+        clear(unmodified_input, start, end, options, &colors);
         return (0, false, false, false);
     }
     let mut unparsed = unmodified_input;
@@ -75,14 +73,15 @@ pub fn print_concurrent(
             );
             return if !out.is_empty()
             {
+                let (width, height) = get_terminal_dimensions();
                 let len = no_col(&out, options.color).len();
-                let wrap = (len - 1) / get_terminal_width() + 1;
-                if len > get_terminal_width() * (get_terminal_height() - 1)
+                let wrap = (len - 1) / width + 1;
+                if len > width * (height - 1)
                 {
                     if long_output
                     {
                         print!(
-                            "\n\x1b[G\x1b[J{}{}",
+                            "\x1b[G\n\x1b[J{}{}",
                             out,
                             if options.color { "\x1b[0m" } else { "" }
                         );
@@ -91,7 +90,7 @@ pub fn print_concurrent(
                     else
                     {
                         print!(
-                            "\x1b[J\n\x1b[Gtoo long, will print on enter\x1b[A\x1b[G\x1b[K{}{}{}",
+                            "\x1b[J\x1b[G\ntoo long, will print on enter\x1b[A\x1b[G\x1b[K{}{}{}",
                             prompt(options, &colors),
                             to_output(&unmodified_input[start..end], options.color, &colors),
                             if options.color { "\x1b[0m" } else { "" },
@@ -102,7 +101,7 @@ pub fn print_concurrent(
                 else
                 {
                     print!(
-                        "\n\x1b[G\x1b[J{}{}\x1b[G\x1b[K{}{}{}",
+                        "\x1b[G\n\x1b[J{}{}\x1b[G\x1b[K{}{}{}",
                         out,
                         "\x1b[A".repeat(wrap),
                         prompt(options, &colors),
@@ -145,7 +144,7 @@ pub fn print_concurrent(
         if input.3
         {
             let n = unparsed.iter().position(|c| c == &'=').unwrap_or(0) + 1;
-            let input = unparsed[n..].iter().collect::<String>();
+            let mut input = unparsed[n..].iter().collect::<String>();
             let mut func = unparsed[..n].to_vec();
             let mut func_vars: Vec<(isize, String)> = Vec::new();
             if func.contains(&'(')
@@ -158,8 +157,22 @@ pub fn print_concurrent(
                     func_vars.push((-1, i.iter().collect()));
                 }
             }
+            if input.contains(':')
+            {
+                let inp = input;
+                let mut split = inp.split(':').collect::<Vec<&str>>();
+                input = split.pop().unwrap().to_string();
+                for i in split
+                {
+                    if i.contains('=')
+                    {
+                        let mut split = i.splitn(2, '=');
+                        func_vars.push((-1, split.next().unwrap().to_string()));
+                    }
+                }
+            }
             let out = match input_var(
-                &input.replace('_', &format!("({})", last.iter().collect::<String>())),
+                &input,
                 vars.to_vec(),
                 &mut func_vars,
                 &mut 0,
@@ -178,14 +191,15 @@ pub fn print_concurrent(
                 clearln(unmodified_input, start, end, options, &colors);
                 return (0, false, false, true);
             }
+            let (width, height) = get_terminal_dimensions();
             let len = no_col(&out, options.color).len();
-            let wrap = (len - 1) / get_terminal_width() + 1;
-            return if len > get_terminal_width() * (get_terminal_height() - 1)
+            let wrap = (len - 1) / width + 1;
+            return if len > width * (height - 1)
             {
                 if long_output
                 {
                     print!(
-                        "\n\x1b[G\x1b[J{}{}",
+                        "\x1b[G\n\x1b[J{}{}",
                         out,
                         if options.color { "\x1b[0m" } else { "" }
                     );
@@ -194,7 +208,7 @@ pub fn print_concurrent(
                 else
                 {
                     print!(
-                        "\x1b[J\n\x1b[Gtoo long, will print on enter\x1b[A\x1b[G\x1b[K{}{}{}",
+                        "\x1b[J\x1b[G\ntoo long, will print on enter\x1b[A\x1b[G\x1b[K{}{}{}",
                         prompt(options, &colors),
                         to_output(&unmodified_input[start..end], options.color, &colors),
                         if options.color { "\x1b[0m" } else { "" },
@@ -205,7 +219,7 @@ pub fn print_concurrent(
             else
             {
                 print!(
-                    "\n\x1b[G\x1b[J{}{}\x1b[G\x1b[K{}{}{}",
+                    "\x1b[G\n\x1b[J{}{}\x1b[G\x1b[K{}{}{}",
                     out,
                     "\x1b[A".repeat(wrap),
                     prompt(options, &colors),
@@ -227,7 +241,7 @@ pub fn print_concurrent(
                 if split.clone().count() > 6
                 {
                     print!(
-                        "\n\x1b[G\x1b[Jtoo many graphs\x1b[A\x1b[G\x1b[K{}{}{}",
+                        "\x1b[G\n\x1b[Jtoo many graphs\x1b[A\x1b[G\x1b[K{}{}{}",
                         prompt(options, &colors),
                         to_output(&unmodified_input[start..end], options.color, &colors),
                         if options.color { "\x1b[0m" } else { "" }
@@ -293,6 +307,7 @@ pub fn print_concurrent(
                 }
             }
             out.pop();
+            let (width, height) = get_terminal_dimensions();
             let no_col = no_col(&out, options.color);
             let wrap = no_col
                 .split(|c| c == &'\n')
@@ -303,18 +318,18 @@ pub fn print_concurrent(
                     }
                     else
                     {
-                        (i.len() - 1) / get_terminal_width() + 1
+                        (i.len() - 1) / width + 1
                     }
                 })
                 .sum();
             let len = no_col.len();
-            let out = out.replace('\n', "\n\x1b[G");
-            if len > get_terminal_width() * (get_terminal_height() - 1)
+            let out = out.replace('\n', "\x1b[G\n");
+            if len > width * (height - 1)
             {
                 if long_output
                 {
                     print!(
-                        "\n\x1b[G\x1b[J{}{}",
+                        "\x1b[G\n\x1b[J{}{}",
                         out,
                         if options.color { "\x1b[0m" } else { "" }
                     );
@@ -323,7 +338,7 @@ pub fn print_concurrent(
                 else
                 {
                     print!(
-                        "\x1b[J\n\x1b[Gtoo long, will print on enter\x1b[A\x1b[G\x1b[K{}{}{}",
+                        "\x1b[J\x1b[G\ntoo long, will print on enter\x1b[A\x1b[G\x1b[K{}{}{}",
                         prompt(options, &colors),
                         to_output(&unmodified_input[start..end], options.color, &colors),
                         if options.color { "\x1b[0m" } else { "" },
@@ -334,7 +349,7 @@ pub fn print_concurrent(
             else
             {
                 print!(
-                    "\n\x1b[G\x1b[J{}{}\x1b[G\x1b[K{}{}{}",
+                    "\x1b[G\n\x1b[J{}{}\x1b[G\x1b[K{}{}{}",
                     out,
                     "\x1b[A".repeat(wrap),
                     prompt(options, &colors),
@@ -354,14 +369,15 @@ pub fn print_concurrent(
                 &input,
                 &last.iter().collect::<String>(),
             );
+            let (width, height) = get_terminal_dimensions();
             let len = no_col(&out, options.color).len();
-            let wrap = (len - 1) / get_terminal_width() + 1;
-            if len > get_terminal_width() * (get_terminal_height() - 1)
+            let wrap = (len - 1) / width + 1;
+            if len > width * (height - 1)
             {
                 if long_output
                 {
                     print!(
-                        "\n\x1b[G\x1b[J{}{}",
+                        "\x1b[G\n\x1b[J{}{}",
                         out,
                         if options.color { "\x1b[0m" } else { "" }
                     );
@@ -370,7 +386,7 @@ pub fn print_concurrent(
                 else
                 {
                     print!(
-                        "\x1b[J\n\x1b[Gtoo long, will print on enter\x1b[A\x1b[G\x1b[K{}{}{}",
+                        "\x1b[J\x1b[G\ntoo long, will print on enter\x1b[A\x1b[G\x1b[K{}{}{}",
                         prompt(options, &colors),
                         to_output(&unmodified_input[start..end], options.color, &colors),
                         if options.color { "\x1b[0m" } else { "" },
@@ -381,7 +397,7 @@ pub fn print_concurrent(
             else
             {
                 print!(
-                    "\n\x1b[G\x1b[J{}{}\x1b[G\x1b[K{}{}{}",
+                    "\x1b[G\n\x1b[J{}{}\x1b[G\x1b[K{}{}{}",
                     out,
                     "\x1b[A".repeat(wrap),
                     prompt(options, &colors),
@@ -397,7 +413,14 @@ pub fn print_concurrent(
         Ok(n) => n,
         Err(s) =>
         {
-            handle_err(s, unmodified_input, options, &colors, start, end);
+            if s == " "
+            {
+                clear(unmodified_input, start, end, options, &colors)
+            }
+            else
+            {
+                handle_err(s, unmodified_input, options, &colors, start, end);
+            }
             return (0, false, false, false);
         }
     };
@@ -510,7 +533,7 @@ pub fn print_concurrent(
         {
             (String::new(), String::new())
         };
-        let terlen = get_terminal_width();
+        let (width, height) = get_terminal_dimensions();
         let len1 = no_col(&output.0, options.color).len();
         let len2 = no_col(&output.1, options.color).len();
         if (frac == 1 && !options.frac)
@@ -523,29 +546,21 @@ pub fn print_concurrent(
                 {
                     0
                 })
-                > terlen
+                > width
         {
             frac = 0;
         }
-        if len1 + len2 > terlen * (get_terminal_height() - 1)
+        if len1 + len2 > width * (height - 1)
         {
             if long_output
             {
-                let num = len1.div_ceil(terlen)
-                    + if len2 != 0
-                    {
-                        (len2 - 1).div_ceil(terlen) - 1
-                    }
-                    else
-                    {
-                        0
-                    }
-                    - 1;
+                let num = len1.div_ceil(width).saturating_sub(1)
+                    + len2.saturating_sub(1).div_ceil(width).saturating_sub(1);
                 print!(
-                    "\x1b[J{}\n\x1b[G\x1b[K{}{}{}{}",
+                    "\x1b[G\x1b[J{}\x1b[G\n{}{}{}{}",
                     if frac == 1
                     {
-                        format!("\n\x1b[G\x1b[K{}{}", frac_a, frac_b)
+                        format!("\x1b[G\n{}{}", frac_a, frac_b)
                     }
                     else
                     {
@@ -554,7 +569,7 @@ pub fn print_concurrent(
                     output.0,
                     if len1 != 0 && len2 != 0
                     {
-                        "\n\x1b[G"
+                        "\x1b[G\n"
                     }
                     else
                     {
@@ -568,10 +583,10 @@ pub fn print_concurrent(
             else
             {
                 print!(
-                    "\x1b[J{}\n\x1b[Gtoo long, will print on enter{}\x1b[A\x1b[G\x1b[K{}{}{}",
+                    "\x1b[G\x1b[J{}\x1b[G\ntoo long, will print on enter{}\x1b[A\x1b[G{}{}{}",
                     if frac == 1
                     {
-                        format!("\n\x1b[G\x1b[K{}{}", frac_a, frac_b)
+                        format!("\x1b[G\n{}{}", frac_a, frac_b)
                     }
                     else
                     {
@@ -585,23 +600,15 @@ pub fn print_concurrent(
                 long = true;
             }
         }
-        else if len1 + len2 > terlen
+        else if len1 + len2 > width
         {
-            let num = len1.div_ceil(terlen)
-                + if len2 != 0
-                {
-                    (len2 - 1).div_ceil(terlen) - 1
-                }
-                else
-                {
-                    0
-                }
-                - 1;
+            let num = len1.div_ceil(width).saturating_sub(1)
+                + len2.saturating_sub(1).div_ceil(width).saturating_sub(1);
             print!(
-                "\x1b[J{}\n\x1b[G\x1b[K{}{}{}{}\x1b[A\x1b[A\x1b[G\x1b[K{}{}{}",
+                "\x1b[G\x1b[J{}\x1b[G\n{}{}{}{}\x1b[A\x1b[A\x1b[G{}{}{}",
                 if frac == 1
                 {
-                    format!("\n\x1b[G\x1b[K{}{}", frac_a, frac_b)
+                    format!("\x1b[G\n{}{}", frac_a, frac_b)
                 }
                 else
                 {
@@ -610,14 +617,16 @@ pub fn print_concurrent(
                 output.0,
                 if len1 != 0 && len2 != 0
                 {
-                    "\n\x1b[G"
+                    "\x1b[G\n"
                 }
                 else
                 {
                     ""
                 },
                 &output.1.replace('+', ""),
-                "\x1b[A".repeat(num + frac - if len1 == 0 || len2 == 0 { 1 } else { 0 }),
+                "\x1b[A".repeat(
+                    (num + frac).saturating_sub(if len1 == 0 || len2 == 0 { 1 } else { 0 })
+                ),
                 prompt(options, &colors),
                 to_output(&unmodified_input[start..end], options.color, &colors),
                 if options.color { "\x1b[0m" } else { "" },
@@ -627,10 +636,12 @@ pub fn print_concurrent(
         else
         {
             print!(
-                "\x1b[J{}\n\x1b[G\x1b[K{}{}\x1b[A{}\x1b[G\x1b[K{}{}{}",
+                "\x1b[G{}{}\x1b[K{}\x1b[G\n{}{}\x1b[J{}\x1b[A\x1b[G{}{}",
+                prompt(options, &colors),
+                to_output(&unmodified_input[start..end], options.color, &colors),
                 if frac == 1
                 {
-                    format!("\n\x1b[G\x1b[K{}{}", frac_a, frac_b)
+                    format!("\x1b[G\n{}{}\x1b[K", frac_a, frac_b)
                 }
                 else
                 {
@@ -639,10 +650,9 @@ pub fn print_concurrent(
                 output.0,
                 output.1,
                 if frac == 1 { "\x1b[A" } else { "" },
-                prompt(options, &colors),
-                to_output(&unmodified_input[start..end], options.color, &colors),
+                "\x1b[C".repeat(if options.prompt { 2 } else { 0 } + (end - start)),
                 if options.color { "\x1b[0m" } else { "" }
-            );
+            )
         }
     }
     else if let Vector(mut v) = num
@@ -653,15 +663,9 @@ pub fn print_concurrent(
                 v,
                 match options.deg
                 {
-                    Degrees =>
-                    {
-                        Complex::with_val(options.prec, 180) / Complex::with_val(options.prec, Pi)
-                    }
+                    Degrees => 180 / Complex::with_val(options.prec, Pi),
                     Radians => Complex::with_val(options.prec, 1),
-                    Gradians =>
-                    {
-                        Complex::with_val(options.prec, 200) / Complex::with_val(options.prec, Pi)
-                    }
+                    Gradians => 200 / Complex::with_val(options.prec, Pi),
                 },
             );
         }
@@ -737,28 +741,28 @@ pub fn print_concurrent(
                 frac_out += ",";
             }
         }
-        let terlen = get_terminal_width();
+        let (width, height) = get_terminal_dimensions();
         let length = no_col(&output, options.color).len();
         if frac_out != output
         {
             frac = 1;
         }
         if (frac == 1 && !options.frac)
-            || no_col(&frac_out, options.color).len() > terlen
-            || length > terlen
+            || no_col(&frac_out, options.color).len() > width
+            || length > width
         {
             frac = 0;
         }
-        if length > terlen * (get_terminal_height() - 1)
+        if length > width * (height - 1)
         {
             if long_output
             {
-                let num = (length - 1) / terlen;
+                let num = (length - 1) / width;
                 print!(
-                    "\x1b[J{}\n\x1b[G\x1b[K{}{}",
+                    "\x1b[G\x1b[J{}\x1b[G\n{}{}",
                     if frac == 1 && options.frac
                     {
-                        format!("\n\x1b[G\x1b[K{}", frac_out)
+                        format!("\x1b[G\n{}", frac_out)
                     }
                     else
                     {
@@ -772,7 +776,7 @@ pub fn print_concurrent(
             else
             {
                 print!(
-                    "\x1b[J\n\x1b[Gtoo long, will print on enter\x1b[A\x1b[G\x1b[K{}{}{}",
+                    "\x1b[G\x1b[J\ntoo long, will print on enter\x1b[A\x1b[G{}{}{}",
                     prompt(options, &colors),
                     to_output(&unmodified_input[start..end], options.color, &colors),
                     if options.color { "\x1b[0m" } else { "" },
@@ -783,12 +787,14 @@ pub fn print_concurrent(
         }
         else
         {
-            let num = (length - 1) / terlen;
+            let num = (length - 1) / width;
             print!(
-                "\x1b[J{}\n\x1b[G\x1b[K{}{}\x1b[A{}\x1b[G\x1b[K{}{}{}",
-                if frac == 1 && options.frac
+                "\x1b[G{}{}\x1b[K{}\x1b[G\n{}\x1b[J{}{}\x1b[A\x1b[G{}{}",
+                prompt(options, &colors),
+                to_output(&unmodified_input[start..end], options.color, &colors),
+                if frac == 1
                 {
-                    format!("\n\x1b[G\x1b[K{}", frac_out)
+                    format!("\x1b[G\n{}\x1b[K", frac_out)
                 }
                 else
                 {
@@ -796,16 +802,8 @@ pub fn print_concurrent(
                 },
                 output,
                 "\x1b[A".repeat(num),
-                if frac == 1 && options.frac
-                {
-                    "\x1b[A"
-                }
-                else
-                {
-                    ""
-                },
-                prompt(options, &colors),
-                to_output(&unmodified_input[start..end], options.color, &colors),
+                if frac == 1 { "\x1b[A" } else { "" },
+                "\x1b[C".repeat(if options.prompt { 2 } else { 0 } + (end - start)),
                 if options.color { "\x1b[0m" } else { "" }
             );
             frac += num;
@@ -908,8 +906,8 @@ pub fn print_concurrent(
                 if options.multi
                 {
                     num += 1;
-                    output += "\n\x1b[G";
-                    frac_out += "\n\x1b[G";
+                    output += "\x1b[K\x1b[G\n";
+                    frac_out += "\x1b[K\x1b[G\n";
                 }
                 else
                 {
@@ -923,7 +921,7 @@ pub fn print_concurrent(
             output += "}";
             frac_out += "}";
         }
-        let terlen = get_terminal_width();
+        let (width, height) = get_terminal_dimensions();
         let length = no_col(&output, options.color).len() - 1;
         if frac_out != output
         {
@@ -931,10 +929,10 @@ pub fn print_concurrent(
         }
         if !options.multi
         {
-            num += (length - 1) / terlen;
+            num += (length - 1) / width;
             if (frac == 1 && !options.frac)
-                || no_col(&frac_out, options.color).len() > terlen
-                || length > terlen
+                || no_col(&frac_out, options.color).len() > width
+                || length > width
             {
                 frac = 0;
             }
@@ -949,7 +947,7 @@ pub fn print_concurrent(
                 {
                     len = 0
                 }
-                else if len > terlen
+                else if len > width
                 {
                     frac = 0;
                     break;
@@ -963,20 +961,20 @@ pub fn print_concurrent(
                 {
                     len = 0
                 }
-                else if len > terlen
+                else if len > width
                 {
                     len = 0;
                     num += 1;
                 }
             }
-            frac_out += "\n";
+            frac_out += "\x1b[K\n";
         }
-        if length > terlen * (get_terminal_height() - 1) || num > (get_terminal_height() - 2)
+        if length > width * (height - 1) || num > (height - 2)
         {
             if long_output
             {
                 print!(
-                    "\x1b[J{}\n\x1b[G\x1b[K{}{}",
+                    "\x1b[G\x1b[J{}\x1b[G\n{}{}",
                     if frac == 1 && options.frac
                     {
                         num *= 2;
@@ -984,7 +982,7 @@ pub fn print_concurrent(
                         {
                             num += 1;
                         }
-                        format!("\n\x1b[G\x1b[K{}", frac_out)
+                        format!("\x1b[G\n{}", frac_out)
                     }
                     else
                     {
@@ -998,7 +996,7 @@ pub fn print_concurrent(
             else
             {
                 print!(
-                    "\x1b[J\n\x1b[Gtoo long, will print on enter\x1b[A\x1b[G\x1b[K{}{}{}",
+                    "\x1b[G\x1b[J\ntoo long, will print on enter\x1b[A\x1b[G{}{}{}",
                     prompt(options, &colors),
                     to_output(&unmodified_input[start..end], options.color, &colors),
                     if options.color { "\x1b[0m" } else { "" },
@@ -1010,15 +1008,17 @@ pub fn print_concurrent(
         else
         {
             print!(
-                "\x1b[J{}\n\x1b[G\x1b[K{}{}\x1b[A{}\x1b[G\x1b[K{}{}{}",
-                if frac == 1 && options.frac
+                "\x1b[G{}{}\x1b[K{}\x1b[G\n{}\x1b[J{}{}\x1b[A\x1b[G{}{}",
+                prompt(options, &colors),
+                to_output(&unmodified_input[start..end], options.color, &colors),
+                if frac == 1
                 {
                     num *= 2;
                     if options.multi
                     {
                         num += 1;
                     }
-                    format!("\n\x1b[G\x1b[K{}", frac_out)
+                    format!("\x1b[G\n{}\x1b[K", frac_out)
                 }
                 else
                 {
@@ -1026,16 +1026,8 @@ pub fn print_concurrent(
                 },
                 output,
                 "\x1b[A".repeat(num),
-                if frac == 1 && options.frac
-                {
-                    "\x1b[A"
-                }
-                else
-                {
-                    ""
-                },
-                prompt(options, &colors),
-                to_output(&unmodified_input[start..end], options.color, &colors),
+                if frac == 1 { "\x1b[A" } else { "" },
+                "\x1b[C".repeat(if options.prompt { 2 } else { 0 } + (end - start)),
                 if options.color { "\x1b[0m" } else { "" }
             );
             frac += num;
@@ -1071,15 +1063,9 @@ pub fn print_answer(num: NumStr, options: Options, colors: &Colors)
                 v,
                 match options.deg
                 {
-                    Degrees =>
-                    {
-                        Complex::with_val(options.prec, 180) / Complex::with_val(options.prec, Pi)
-                    }
+                    Degrees => 180 / Complex::with_val(options.prec, Pi),
                     Radians => Complex::with_val(options.prec, 1),
-                    Gradians =>
-                    {
-                        Complex::with_val(options.prec, 200) / Complex::with_val(options.prec, Pi)
-                    }
+                    Gradians => 200 / Complex::with_val(options.prec, Pi),
                 },
             );
         }
@@ -1419,9 +1405,10 @@ fn to_string(num: &Float, decimals: usize, imag: bool) -> String
         };
     }
     let exp = exp.unwrap();
-    let decimals = if decimals == usize::MAX - 1 && (get_terminal_width() as i32) > (2i32 + exp)
+    let width = get_terminal_dimensions().0;
+    let decimals = if decimals == usize::MAX - 1 && (width as i32) > (2i32 + exp)
     {
-        (get_terminal_width() as i32
+        (width as i32
             - match exp.cmp(&0)
             {
                 Ordering::Equal => 2i32,
@@ -1609,7 +1596,7 @@ fn remove_trailing_zeros(input: &str, dec: usize, prec: (u32, u32)) -> String
     };
     let dec = if dec == usize::MAX - 1
     {
-        get_terminal_width()
+        get_terminal_dimensions().0
             - (if &input[pos..] == "e0"
             {
                 2
@@ -1670,4 +1657,4 @@ fn remove_trailing_zeros(input: &str, dec: usize, prec: (u32, u32)) -> String
                 .to_string()
         }
     }
-}
\ No newline at end of file
+}
